@@ -40,13 +40,6 @@ struct counter_map
 	unsigned int mappos;
 };
 
-/* Convenience structures */
-struct arpt_error_target
-{
-	STRUCT_ENTRY_TARGET t;
-	char error[TABLE_MAXNAMELEN];
-};
-
 struct chain_cache
 {
 	char name[TABLE_MAXNAMELEN];
@@ -198,19 +191,16 @@ alloc_handle(const char *tablename, unsigned int size, unsigned int num_rules)
 		+ size
 		+ num_rules * sizeof(struct counter_map);
 
-	if ((h = malloc(len)) == NULL) {
+	if ((h = calloc(1, len)) == NULL) {
 		errno = ENOMEM;
 		return NULL;
 	}
 
-	h->changed = 0;
-	h->cache_num_chains = 0;
-	h->cache_chain_heads = NULL;
 	h->counter_map = (void *)h
 		+ sizeof(STRUCT_TC_HANDLE)
 		+ size;
-	strcpy(h->info.name, tablename);
-	strcpy(h->entries.name, tablename);
+	strncpy(h->info.name, tablename, sizeof(h->info.name) - 1);
+	strncpy(h->entries.name, tablename, sizeof(h->entries.name) - 1);
 
 	return h;
 }
@@ -255,22 +245,7 @@ TC_INIT(const char *tablename)
 	    == NULL)
 		return NULL;
 
-/* Too hard --RR */
-#if 0
-	sprintf(pathname, "%s/%s", ARPT_LIB_DIR, info.name);
-	dynlib = dlopen(pathname, RTLD_NOW);
-	if (!dynlib) {
-		errno = ENOENT;
-		return NULL;
-	}
-	h->hooknames = dlsym(dynlib, "hooknames");
-	if (!h->hooknames) {
-		errno = ENOENT;
-		return NULL;
-	}
-#else
 	h->hooknames = hooknames;
-#endif
 
 	/* Initialize current state */
 	h->info = info;
@@ -357,8 +332,9 @@ add_chain(STRUCT_ENTRY *e, TC_HANDLE_T h, STRUCT_ENTRY **prev)
 		h->cache_chain_heads[h->cache_num_chains-1].end
 			= *prev;
 
-		strcpy(h->cache_chain_heads[h->cache_num_chains].name,
-		       (const char *)GET_TARGET(e)->data);
+		strncpy(h->cache_chain_heads[h->cache_num_chains].name,
+		       (const char *)GET_TARGET(e)->data, TABLE_MAXNAMELEN-1);
+		h->cache_chain_heads[h->cache_num_chains].name[TABLE_MAXNAMELEN-1] = '\0';
 		h->cache_chain_heads[h->cache_num_chains].start
 			= (void *)e + e->next_offset;
 		h->cache_num_chains++;
@@ -368,8 +344,9 @@ add_chain(STRUCT_ENTRY *e, TC_HANDLE_T h, STRUCT_ENTRY **prev)
 			h->cache_chain_heads[h->cache_num_chains-1].end
 				= *prev;
 
-		strcpy(h->cache_chain_heads[h->cache_num_chains].name,
-		       h->hooknames[builtin-1]);
+		strncpy(h->cache_chain_heads[h->cache_num_chains].name,
+		       h->hooknames[builtin-1], TABLE_MAXNAMELEN-1);
+		h->cache_chain_heads[h->cache_num_chains].name[TABLE_MAXNAMELEN-1] = '\0';
 		h->cache_chain_heads[h->cache_num_chains].start
 			= (void *)e;
 		h->cache_num_chains++;
@@ -890,7 +867,7 @@ map_target(const TC_HANDLE_T handle,
 	/* memset to all 0 for your memcmp convenience. */
 	memset(t->u.user.name + strlen(t->u.user.name),
 	       0,
-	       FUNCTION_MAXNAMELEN - strlen(t->u.user.name));
+	       FUNCTION_MAXNAMELEN - 1 - strlen(t->u.user.name));
 	return 1;
 }
 
@@ -1353,9 +1330,9 @@ TC_CREATE_CHAIN(const ARPT_CHAINLABEL chain, TC_HANDLE_T *handle)
 	newc.head.next_offset
 		= sizeof(STRUCT_ENTRY)
 		+ ALIGN(sizeof(struct arpt_error_target));
-	strcpy(newc.name.t.u.user.name, ERROR_TARGET);
-	newc.name.t.u.target_size = ALIGN(sizeof(struct arpt_error_target));
-	strcpy(newc.name.error, chain);
+	strcpy(newc.name.target.u.user.name, ERROR_TARGET);
+	newc.name.target.u.target_size = ALIGN(sizeof(struct arpt_error_target));
+	strcpy(newc.name.errorname, chain);
 
 	newc.ret.target_offset = sizeof(STRUCT_ENTRY);
 	newc.ret.next_offset
@@ -1493,8 +1470,8 @@ int TC_RENAME_CHAIN(const ARPT_CHAINLABEL oldname,
 	t = (struct arpt_error_target *)
 		GET_TARGET(get_entry(*handle, labeloff));
 
-	memset(t->error, 0, sizeof(t->error));
-	strcpy(t->error, newname);
+	memset(t->errorname, 0, sizeof(t->errorname));
+	strcpy(t->errorname, newname);
 	set_changed(*handle);
 
 	return 1;
@@ -1706,7 +1683,7 @@ TC_COMMIT(TC_HANDLE_T *handle)
 		/* Kernel will think that pointer should be 64-bits, and get
 		   padding.  So we accomodate here (assumption: alignment of
 		   `counters' is on 64-bit boundary). */
-		u_int64_t *kernptr = (u_int64_t *)&newcounters->counters;
+		uint64_t *kernptr = (uint64_t *)&newcounters->counters;
 		if ((unsigned long)&newcounters->counters % 8 != 0) {
 			fprintf(stderr,
 				"counters alignment incorrect! Mail rusty!\n");
